"""
OpenXBOW extractor
==================

This module contains the OpenXBOW extractor, which is a wrapper around
the OpenXBOW Java program.

.. autosummary::
    :toctree:

    OpenXBOWExtractor
    OpenXBOWExtractorConfig
"""

from .openxbow import OpenXBOWExtractor, OpenXBOWExtractorConfig
