"""
PyTorch backend
===============

This package contains the PyTorch backend for the ERTK.

.. autosummary::
    :toctree: generated/

    models
    train
    dataset
    classification
    utils
"""

from .models import get_pt_model, get_pt_model_fn
