"""
Audioset feature extractors
===========================

This module contains feature extractors for the Audioset dataset.

.. autosummary::
    :toctree:

    VGGishExtractor
    VGGishExtractorConfig
    YamnetExtractor
    YamnetExtractorConfig
"""

from .audioset import (
    VGGishExtractor,
    VGGishExtractorConfig,
    YamnetExtractor,
    YamnetExtractorConfig,
)
